﻿using UnityEngine;
using System.Collections;

public class CameraController : MonoBehaviour
{
  #region vars
  public float rotSpeed = 50;
  public float moveSpeed;
  public float rotDamp;
  public float moveDamp;

  public float height = 0;
  public float maxHeight = 200;

  public float radius;

  public Transform Camera;
  public Transform planet;

  protected Transform trans;
  protected Transform parent;

  protected float angle = 90f;
  protected Vector3 direction;
  protected Quaternion rotation = Quaternion.identity;

  #endregion

  #region Unity methods
  void Start()
  {
    trans = transform;
    parent = transform.parent;
  }

  float lastMoveTime = 0;

  void Update()
  {
    //parent.position = planet.position; // If you want to have a moving planet

    direction = new Vector3(Mathf.Sin(angle), Mathf.Cos(angle));

    height -= Input.GetAxis("Mouse ScrollWheel") * 50;
    height = Mathf.Clamp(height, 0, maxHeight);

    if (Time.time > lastMoveTime + 5)
    {
      // eww fix this
      var ang = Mathf.Clamp(25 + (height / maxHeight) * 65, 25, 90);
      Camera.localRotation = Quaternion.Lerp(Camera.localRotation, Quaternion.Euler(ang, 0, 0), Time.fixedDeltaTime * rotDamp * 0.1f);
    }

    if (Input.GetMouseButton(1))
    {
      Rotation(Input.GetAxis("Mouse X") * rotSpeed);

      Camera.Rotate(new Vector3(Input.GetAxis("Mouse Y"), 0, 0));



      /*
        set fixed values instead of in/decrementing
        over time lerp towards a specific angle
        angle depends on height
        
    
        far away = 90
        close = 45

      */
    }

    if (Input.GetKey(KeyCode.LeftShift))
      Position(Input.GetAxis("Horizontal") * -moveSpeed, 0);
    else
      Rotation(Input.GetAxis("Horizontal") * -rotSpeed);


    Position(0, Input.GetAxis("Vertical") * moveSpeed);

    Movement();
  }
  #endregion

  #region Actions
  protected void Rotation(float amt)
  {
    if (amt > 0)
      lastMoveTime = Time.time;

    angle += amt * Mathf.Deg2Rad * Time.fixedDeltaTime;
  }

  protected void Position(float x, float y)
  {
    if (x > 0 || y > 0)
      lastMoveTime = Time.time;

    Vector2 perpendicular = new Vector2(-direction.y, direction.x);
    Quaternion vRot = Quaternion.AngleAxis(y, perpendicular);
    Quaternion hRot = Quaternion.AngleAxis(x, direction);
    rotation *= hRot * vRot;
  }

  protected void Movement()
  {
    trans.localPosition = Vector3.Lerp(trans.localPosition, rotation * Vector3.forward * GetHeight(), Time.fixedDeltaTime * moveDamp);
    trans.rotation = Quaternion.Lerp(trans.rotation, rotation * Quaternion.LookRotation(direction, Vector3.forward), Time.fixedDeltaTime * rotDamp);
  }

  protected float GetHeight()
  {
    Ray ray = new Ray(trans.position, planet.position - trans.position);
    RaycastHit hit;

    if (Physics.Raycast(ray, out hit))
    {
      radius = Vector3.Distance(planet.position, hit.point);
    }

    return radius + height;
  }

  #endregion
}
